// DIBitmap.cpp : implementation file
//

#include "stdafx.h"
#include "DIBitmap.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDIBitmap

CDIBitmap::CDIBitmap()
{
    m_pBMI = NULL;
    m_pBits = NULL;
}

CDIBitmap::~CDIBitmap()
{
    //  
    if (m_pBMI != NULL)	 free(m_pBMI);
    if (m_pBits != NULL) free(m_pBits);
}

/////////////////////////////////////////////////////////////////////////////
// Private functions

BOOL CDIBitmap::IsPMDIB(BITMAPINFOHEADER* pBIH)
{
//  TRUE     
// pBIH   Presentation Manager.
    ASSERT(pBIH);
    if (((BITMAPCOREHEADER*)pBIH)->bcSize == sizeof(BITMAPCOREHEADER))
        return TRUE;
    
    return FALSE;
}

int CDIBitmap::GetColorEntriesInDIB(BITMAPINFO* pBmpInfo) 
{
    BITMAPINFOHEADER* pBIH;
    BITMAPCOREHEADER* pBCH;
    int iColors = 0, iBitsPerPixel;

    ASSERT(pBmpInfo);

    pBIH = &(pBmpInfo->bmiHeader);
    pBCH = (BITMAPCOREHEADER*) pBIH;

	//  Windows DIB,      
	// biClrUsed,       
    if ((!IsPMDIB(pBIH)) && (pBIH->biClrUsed != 0))
   	{
        iColors = pBIH->biClrUsed;
		return iColors;
   	}

    //         
    //    
    if (IsPMDIB(pBIH))
    	iBitsPerPixel = pBCH->bcBitCount;
    else 
        iBitsPerPixel = pBIH->biBitCount;

	if (iBitsPerPixel == 1)	iColors = 2;
	if (iBitsPerPixel == 4)	iColors = 16;
	if (iBitsPerPixel == 8)	iColors = 256;

	return iColors;
}

BOOL CDIBitmap::LoadRis(int index)
{
	CFile f;
	if (f.Open(GetFileName(index),CFile::modeRead))
		return LoadDIB(&f);
	else
		return FALSE;
}

BOOL CDIBitmap::LoadDIB(CFile* fRis)
{
    BOOL bIsPM = FALSE;   //  " PM?"
    BITMAPINFO* pBmpInfo = NULL;
    BYTE* pBits = NULL;
	int iColors,          //  
		iColorTableSize,  //   
		iBISize,          //  BITMAPINFO   
		iBitsSize;        //   

    //    
    DWORD dwFileStart = fRis->GetPosition();

    //   .       
    //      
    BITMAPFILEHEADER BmpFileHdr;
    int iBytes;
    iBytes = fRis->Read(&BmpFileHdr, sizeof(BITMAPFILEHEADER));
    if (iBytes != sizeof(BITMAPFILEHEADER))
	{
        AfxMessageBox("Error reading file header");
        goto error;
    }
    //    "BM"    bitmap?
    if (BmpFileHdr.bfType != 0x4D42)
	{
        AfxMessageBox("  bmp ");
        goto error;
    }
    //       Windows DIB
    //   BITMAPINFOHEADER.    PM DIB
    //    
    BITMAPINFOHEADER BmpInfoHdr;
    iBytes = fRis->Read(&BmpInfoHdr, sizeof(BITMAPINFOHEADER)); 
    if (iBytes != sizeof(BITMAPINFOHEADER))
	{
        AfxMessageBox("Error reading header information");
        goto error;
    }
    //   Windows DIB?
    if (BmpInfoHdr.biSize != sizeof(BITMAPINFOHEADER)) 
	{
    	//  Windows DIB.     
    	//  PM DIB.
        bIsPM = TRUE;
    	//   PM DIB?
		if (BmpInfoHdr.biSize != sizeof(BITMAPCOREHEADER))
		{
			//!
       		AfxMessageBox(" DIB ");
       		goto error;
       	}	
        //        PM DIB.
	    //     BITMAPCOREHEADER
    	//     BITMAPINFOHEADER
        fRis->Seek(dwFileStart + sizeof(BITMAPFILEHEADER), CFile::begin);
        BITMAPCOREHEADER BmpCoreHdr;
        iBytes = fRis->Read(&BmpCoreHdr, sizeof(BITMAPCOREHEADER)); 
        if (iBytes != sizeof(BITMAPCOREHEADER))
		{
           	AfxMessageBox("   ");
           	goto error;
        }
	
        BmpInfoHdr.biSize = sizeof(BITMAPINFOHEADER);
        BmpInfoHdr.biWidth = (LONG) BmpCoreHdr.bcWidth;
        BmpInfoHdr.biHeight = (LONG) BmpCoreHdr.bcHeight;
        BmpInfoHdr.biPlanes = BmpCoreHdr.bcPlanes;
        BmpInfoHdr.biBitCount = BmpCoreHdr.bcBitCount;
        BmpInfoHdr.biCompression = BI_RGB;
        BmpInfoHdr.biSizeImage = 0;
        BmpInfoHdr.biXPelsPerMeter = 0;
        BmpInfoHdr.biYPelsPerMeter = 0;
        BmpInfoHdr.biClrUsed = 0;
        BmpInfoHdr.biClrImportant = 0;
    }

	//    
    iColors = GetColorEntriesInDIB((LPBITMAPINFO) &BmpInfoHdr);
    iColorTableSize = iColors * sizeof(RGBQUAD);
    
    //    256 (iColors) 
    iBISize = sizeof(BITMAPINFOHEADER) + iColors * sizeof(RGBQUAD);
    iBitsSize = BmpFileHdr.bfSize - BmpFileHdr.bfOffBits;

    //    
    pBmpInfo = (LPBITMAPINFO) malloc(iBISize);
    if (!pBmpInfo)
	{
        AfxMessageBox("    DIB");
        goto error;
    }
    //     
    memcpy(pBmpInfo, &BmpInfoHdr, sizeof(BITMAPINFOHEADER));

    //     
    if (bIsPM == FALSE)
    {
        iBytes = fRis->Read(((LPBYTE) pBmpInfo) + sizeof(BITMAPINFOHEADER),
                             iColorTableSize);
        if (iBytes != iColorTableSize)
 		{
            AfxMessageBox("   ");
            goto error;
        }
	}
	else
	{
        //    PM      Windows DIB
        LPRGBQUAD lpRGB;
        lpRGB = (LPRGBQUAD) ((LPBYTE) pBmpInfo + sizeof(BITMAPINFOHEADER));
        RGBTRIPLE rgbt;
        for (int i = 0; i < iColors; i++)
       	{
            iBytes = fRis->Read(&rgbt, sizeof(RGBTRIPLE));
            if (iBytes != sizeof(RGBTRIPLE))
			{
                AfxMessageBox("    ");
                goto error;
            }
           	lpRGB->rgbBlue = rgbt.rgbtBlue;
            lpRGB->rgbGreen = rgbt.rgbtGreen;
            lpRGB->rgbRed = rgbt.rgbtRed;
            lpRGB->rgbReserved = 0;
            lpRGB++;
       	}
    }

    //    bitmap
    pBits = (BYTE*) malloc(iBitsSize);
    if (pBits == NULL)
 	{
        AfxMessageBox("     DIB");
        goto error;
    }
    
    //       
    fRis->Seek(dwFileStart + BmpFileHdr.bfOffBits, CFile::begin);

    //  
    iBytes = fRis->Read(pBits, iBitsSize);
    if (iBytes != iBitsSize)
	{
        AfxMessageBox("   ");
        goto error;
    }
		//   ,    
		if (m_pBMI != NULL)
    		free(m_pBMI);
		m_pBMI = pBmpInfo; 
		if (m_pBits != NULL)
	    	free(m_pBits);
		m_pBits = pBits;
		fRis->Close();
	return TRUE;
                    
// Oops!  !
error: 
    if (pBmpInfo)
    	free(pBmpInfo);
    if (pBits)
    	free (pBits);
	fRis->Close();
    return FALSE;    
}

RGBQUAD* CDIBitmap::GetBMColorTabPtr()
{
	//      
	return (LPRGBQUAD)(((BYTE*)(m_pBMI)) + sizeof(BITMAPINFOHEADER));
}
   
void CDIBitmap::DrawBitmap(CDC* pDC, int x, int y,
						 int Width, int Height)
{
    //  DIB       (x,y)

	if (m_pBMI == NULL)	return;

    ::StretchDIBits(pDC->GetSafeHdc(),
         x,                          // x  
         y,                          // y  
         Width,                      //   
         Height,                     //   
         0,                          // x  
         0,                          // y  
         m_pBMI->bmiHeader.biWidth,  //   
         m_pBMI->bmiHeader.biHeight, //   
         GetBMBitsPtr(),             //   
         GetBMHdrPtr(),              //    BITMAPINFO
         DIB_RGB_COLORS,             // 
         SRCCOPY);		             //  
}

void CDIBitmap::DrawBitmap(CDC* pDC, int x, int y)
{
	if (m_pBMI == NULL)	return;
	DrawBitmap(pDC,x,y,m_pBMI->bmiHeader.biWidth,
                       m_pBMI->bmiHeader.biHeight);
}

void CDIBitmap::DrawBitmap(CDC* pDC, CRect* rect)
{
	if (m_pBMI == NULL)	return;
	float aspectBM = (float)GetBMHeight()/GetBMWidth();
	float aspectclient = (float)rect->Height()/rect->Width();
	if (aspectBM > aspectclient)
		rect->right = rect->left + (int)(rect->Height()/aspectBM);
	else
		rect->bottom = rect->top + (int)(rect->Width()*aspectBM);
	DrawBitmap(pDC,rect->left,rect->top,rect->Width(),rect->Height());
}

int CDIBitmap::GetNumClrEntries()
{
//     
    return GetColorEntriesInDIB(m_pBMI);
}

int CDIBitmap::GetBMWidth()
{
//    
	return m_pBMI->bmiHeader.biWidth;
} 

int CDIBitmap::GetBMHeight()
{
//    
	return m_pBMI->bmiHeader.biHeight;
}

CString CDIBitmap::GetFileName(BYTE bIndex) const
{
	CString str;
	str.Format("Ris\\Ris%02u.bmp",bIndex);
	return str;
}
