// VGKDoc.cpp : implementation of the CVGKDoc class
//

#include "stdafx.h"
#include "VGK.h"
#include "VGKDoc.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVGKDoc

IMPLEMENT_DYNCREATE(CVGKDoc, CDocument)

BEGIN_MESSAGE_MAP(CVGKDoc, CDocument)
	//{{AFX_MSG_MAP(CVGKDoc)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVGKDoc construction/destruction

CVGKDoc::CVGKDoc()
{
	// TODO: add one-time construction code here
	m_L = m_T = m_Q = m_Nmax = m_Nmin = m_D0 = m_R2 = m_Lambda = 0.0f;
	m_I = 0;

	m_D1 = m_Alfa = m_R1 = 0.0f;
	m_zi = m_z = 0;
	m_dkv = m_Pst_dop = m_Qst_dop = m_n = m_K = m_P_dop = m_Q_dop = m_ro = m_KPD = 0.0f;
	m_zr = 0;
	m_deltamin = m_deltamax = m_F = m_deltav = m_Pnmin_dop = m_Pnmax_dop =
	m_Qnmin_dop = m_Qnmax_dop = m_deltanmin = m_deltanmax = m_deltaD1min = m_deltaD1max = 0.0f;
	m_zr_ = 0;
	m_deltamin_ = m_deltamax_ = m_Q1pn = m_P1pn = m_P2pn = m_K_KPD = m_KPD_ =
	m_Qtmax = m_Qtmin = m_Txmax = m_Txmin = 0.0f;
}

CVGKDoc::~CVGKDoc()
{
}

/////////////////////////////////////////////////////////////////////////////
// CVGKDoc diagnostics

#ifdef _DEBUG
void CVGKDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CVGKDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

BOOL CVGKDoc::Calculate()
{
	if (m_L <= 0.0f) return FALSE;
	if (m_Nmax <= 0.0f) return FALSE;
	if (m_Nmin <= 0.0f) return FALSE;
	if (m_Q <= 0.0f) return FALSE;
	if (m_T <= 0.0f) return FALSE;
	if (m_I <= 0) return FALSE;
	if (m_D0 <= 0.0f) return FALSE;
	if (m_R2 <= 0.0f) return FALSE;
	if (m_Lambda <= 0.0f) return FALSE;

	const float PI = 3.1415926f;
	const float E = 2.1e+5f; // 
	float Lambda = m_Lambda*PI/180;

	m_D1 = 0.6f*m_T;
	m_Alfa = PI/4;
	m_R1 = 0.96f*m_R2;
	m_zi = (int)(PI*m_D0/m_D1) - 5;
	if (m_zi<=0) return FALSE;
	m_z = m_zi*m_I;
	m_dkv = m_D0 - 2*m_R1*(float)cos(m_Alfa);
	if (m_dkv<=0.0f) return FALSE;
	m_Pst_dop = 20*m_D1*m_D1;
	m_Qst_dop = 10*m_z*m_D1*m_D1;
	m_n = (m_Nmax + m_Nmin)/2;
	m_K = (float)(0.6*pow(m_n,1.0/3.0));
	if (m_K>1)
	{
		m_P_dop = m_Pst_dop/m_K;
		m_Q_dop = m_Qst_dop/m_K;
	}
	else
	{
		m_P_dop = m_Pst_dop;
		m_Q_dop = m_Qst_dop;
	}
	//     
	m_ro = (float)atan( 0.001/(m_R1*sin(m_Alfa)) );
	m_KPD = (float)(tan(Lambda)/tan(Lambda + m_ro));
	m_zr = (int)(0.7*m_z*sqrt(m_Q/m_Q_dop))+1;
	m_deltamin = (float)(3.8*pow(1/m_D1*(m_Q/(10*m_zr))*(m_Q/(10*m_zr)), 1.0/3.0) );
	m_deltamax = m_deltamin;
	m_F = PI*m_D0*m_D0/4;
	m_deltav = 1000*m_Q*m_L/(E*m_F);
	//     
	m_Pnmin_dop = (float)(0.5*m_Q/(0.7*m_z*sin(m_Alfa)));
	m_Pnmax_dop = m_P_dop*(1-0.55f*m_Q/m_Q_dop);
	if (m_Pnmax_dop<=0.0f) return FALSE;
	m_Qnmin_dop = m_Q_dop*(m_P_dop-m_Pnmax_dop)/(0.55f*m_P_dop);
	if (m_Qnmin_dop<=0.0f) return FALSE;
	m_Qnmax_dop = m_Qnmin_dop*(m_P_dop-m_Pnmin_dop)/(m_P_dop-m_Pnmax_dop);
	if (m_Qnmax_dop<=0.0f) return FALSE;
	if (m_Qnmax_dop>=(0.9f*m_Q_dop)) m_Qnmax_dop = 0.9f*m_Q_dop;
	m_deltanmin = (float)(6.0*pow(m_Pnmin_dop*m_Pnmin_dop/(100*m_D1),1.0/3.0));
	m_deltanmax = (float)(m_deltanmin*pow(m_Pnmax_dop/m_Pnmin_dop,2.0/3.0));
	m_deltaD1min = (float)(2.1*pow(m_Pnmin_dop*m_Pnmin_dop/(100*m_D1),1.0/3.0));
	m_deltaD1max = (float)(m_deltaD1min*pow(m_Pnmax_dop/m_Pnmin_dop,2.0/3.0));
	m_zr_ = (int)(0.7*m_z*sqrt(m_Q/m_Qnmin_dop))+1;
	m_deltamin_ = (float)(1.4/pow(m_D1*m_Pnmax_dop/10,1.0/3.0)*m_Q/(10*m_zr_));
	m_deltamax_ = (float)(m_deltamin_*pow(m_Pnmax_dop/m_Pnmin_dop,2.0/3.0));
	float Pn_dop = m_Pnmax_dop;
	m_Q1pn = (float)(m_Q/(m_zr_*sin(m_Alfa)*cos(Lambda)*Pn_dop));
	m_P1pn = 1+1.8f*m_Q1pn/2.8f;
	m_P2pn = 1-m_Q1pn/2.8f;
	m_K_KPD = (float)((m_P1pn*cos(Lambda+m_ro)-m_P2pn*cos(Lambda-m_ro))*tan(Lambda+m_ro)
		             /(m_P1pn*sin(Lambda+m_ro)-m_P2pn*sin(Lambda-m_ro)));
	m_KPD_ = m_K_KPD*m_KPD;
	m_Qtmax = (float)(m_Pnmax_dop*m_zr_*sin(m_Alfa)*(sin(Lambda+m_ro)-sin(Lambda-m_ro))
                     /(0.43*sin(Lambda+m_ro)+0.55*sin(Lambda-m_ro)));
	m_Qtmin = m_Qtmax*m_Pnmin_dop/m_Pnmax_dop;
	m_Txmax = (float)(m_Pnmax_dop*m_zr_*sin(m_Alfa)*m_dkv/2*(sin(Lambda+m_ro)-sin(Lambda-m_ro)));
	m_Txmin = m_Txmax*m_Pnmin_dop/m_Pnmax_dop;

	return TRUE;
}

CString CVGKDoc::GetOutputStr(int index)
{
	CString t;
	switch (index)
	{
		case  0: t.Format("%s\t%.1f ",outData[index],m_Pst_dop); break;
		case  1: t.Format("%s\t%.1f ",outData[index],m_Qst_dop); break;
		case  2: t.Format("%s\t%.2f",outData[index],m_K); break;
		case  3: t.Format("%s\t%.1f ",outData[index],m_P_dop); break;
		case  4: t.Format("%s\t%.1f ",outData[index],m_Q_dop); break;
		case  5: t.Format("%s\t%.2f",outData[index],m_KPD); break;
		case  6: t.Format("%s\t%.2f",outData[index],m_KPD_); break;
		case  7: t.Format("%s\t%.1f ",outData[index],m_Qnmin_dop); break;
		case  8: t.Format("%s\t%.1f ",outData[index],m_Qnmax_dop); break;
		case  9: t.Format("%s\t%.1f ",outData[index],m_Pnmin_dop); break;
		case 10: t.Format("%s\t%.1f ",outData[index],m_Pnmax_dop); break;
		case 11: t.Format("%s\t%.1f ",outData[index],m_deltanmin); break;
		case 12: t.Format("%s\t%.1f ",outData[index],m_deltanmax); break;
		case 13: t.Format("%s\t%.1f ",outData[index],m_deltaD1min); break;
		case 14: t.Format("%s\t%.1f ",outData[index],m_deltaD1max); break;
		case 15: t.Format("%s\t%.1f ",outData[index],m_deltamin); break;
		case 16: t.Format("%s\t%.1f ",outData[index],m_deltamax); break;
		case 17: t.Format("%s\t%.1f ",outData[index],m_deltamin_); break;
		case 18: t.Format("%s\t%.1f ",outData[index],m_deltamax_); break;
		case 19: t.Format("%s\t%.1f ",outData[index],m_deltav); break;
		case 20: t.Format("%s\t%.1f ",outData[index],m_Qtmin); break;
		case 21: t.Format("%s\t%.1f ",outData[index],m_Qtmax); break;
		case 22: t.Format("%s\t%.1f ͷ",outData[index],m_Txmin); break;
		case 23: t.Format("%s\t%.1f ͷ",outData[index],m_Txmax); break;
	}
	return t;
}
